@@ -130,9 +130,10 @@ class ElasticAPM:
             receive: receive awaitable callable
             send: send awaitable callable
         """
-        # we only handle the http scope, raise an exception for anything else
-        # see https://www.uvicorn.org/#the-asgi-interface
-        assert scope["type"] == "http"
+        # we only handle the http scope, skip anything else.
+        if scope["type"] != "http":
+            await self.app(scope, receive, send)
+            return
 
         @functools.wraps(send)
         async def wrapped_send(message):
